@@ -7,16 +7,16 @@
  * 	for the code
  *
  * This software is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
+ * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
- * Lesser General Public License for more details.
+ *  General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
+ * You should have received a copy of the GNU  General Public
  * License along with this software; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
@@ -114,7 +114,7 @@ get_station_weather_data(const gchar *station_id_with_path, GHashTable *data,
                 if(get_detail_data)
                     days_number = parse_xml_detail_data(buffer, root_node, data);
                 else
-                    days_number = parse_xml_data(buffer, root_node, data);
+                    days_number = parse_xml_data(buffer, doc, data);
             }
             xmlFreeDoc(doc);
             xmlCleanupParser();
@@ -924,6 +924,7 @@ fill_current_data(xmlNode *root_node, GHashTable *current_weather, GHashTable *d
      
                                                 xmlFree(temp_xml_string);
                                                 temp_xml_string = xmlNodeGetContent(child_node7->children);
+                                                fprintf(stderr, "ssssssssssssssssssssssssss %s\n", temp_xml_string);
                                                 tmp_tm = get_date_for_current_weather((char*)temp_xml_string);
                                                 memset(buffer, 0, sizeof(buffer));
                                                 setlocale(LC_TIME, "POSIX");
@@ -951,190 +952,242 @@ fill_current_data(xmlNode *root_node, GHashTable *current_weather, GHashTable *d
 }
 /*******************************************************************************/
 gint
-parse_xml_data(const gchar *station_id, xmlNode *root_node, GHashTable *data){
-    xmlNode     *cur_node = NULL,
-                *child_node = NULL,
-                *child_node3 = NULL;
-    xmlChar     *temp_xml_string = NULL;
-    gint        count_day = 0,
-                count_night = 0,
-                count_of_div = 0,
-                count_of_div_temp = 0,
-                count_of_table = 0;
+parse_xml_data(const gchar *station_id, htmlDocPtr doc, GHashTable *data){
     gchar       buff[256],
-                *delimiter = NULL;
+                buffer[2048];
     struct tm   tmp_tm = {0};
-    struct tm   *gmt = NULL ;
-    time_t      first_day, day_in_list, night_in_list;
     GSList      *forecast = NULL;
     GSList      *tmp = NULL;
     GHashTable  *day = NULL;
     gboolean    flag;
+    gboolean    night_flag;
+    gint        size;
+    gint        i;
     GHashTable *hash_for_translate;
     GHashTable *hash_for_icons;
+    xmlXPathContextPtr xpathCtx; 
+    xmlXPathObjectPtr xpathObj; 
+    xmlXPathObjectPtr xpathObj2; 
+    xmlXPathObjectPtr xpathObj3; 
+    xmlXPathObjectPtr xpathObj4; 
+    xmlXPathObjectPtr xpathObj5; 
+    xmlXPathObjectPtr xpathObj6; 
+    xmlXPathObjectPtr xpathObj7; 
+    xmlXPathObjectPtr xpathObj8; 
+    xmlXPathObjectPtr xpathObj9; 
+    xmlNodeSetPtr nodes;
+    gchar       *temp_char;
+    gint        pressure; 
+    gint        speed;
 
 /* Fix me free memory */
 hash_for_translate = hash_description_gismeteo_table_create();
 hash_for_icons = hash_icons_gismeteo_table_create();
-/* calculate count of day */
-    for(cur_node = root_node->children; cur_node; cur_node = cur_node->next){
-        if( cur_node->type == XML_ELEMENT_NODE ){
-            /* get weather station data */
-            if(!xmlStrcmp(cur_node->name, (const xmlChar *) "body" ) ){
-                for(child_node = cur_node->children; child_node; child_node = child_node->next){
-                    if( child_node->type == XML_ELEMENT_NODE ){
-                        if (!xmlStrcmp(child_node->name, (const xmlChar *)"div") ){
-                            count_of_div ++;
-                            count_of_div_temp ++;
-                        }
-                        if (!xmlStrcmp(child_node->name, (const xmlChar *)"table") )
-                            count_of_table ++;
-                        if (count_of_div == 2 && count_of_table == 0){
-                            temp_xml_string = xmlNodeGetContent(child_node);
-                            if (temp_xml_string){
-                                tmp_tm = get_data_from_russia_data((char*)temp_xml_string);
-                                first_day = mktime(&tmp_tm) - 3600*24;
-                                day_in_list = night_in_list = first_day;
-                                xmlFree(temp_xml_string);
-                            }
-                        }
-                        if (count_of_div > 2 && 
-                            !xmlStrcmp(child_node->name, (const xmlChar *)"div")){
-                            temp_xml_string = xmlGetProp(child_node, (const xmlChar*)"style");
-                            if (!xmlStrcmp(temp_xml_string,
-                                           (const xmlChar*)"float: left; padding: 0 0 0 4px;")){
-                                xmlFree(temp_xml_string);
-                                for(child_node3 = child_node->children; child_node3 != NULL; child_node3 = child_node3->next){
-                                    if (!xmlStrcmp(child_node3->name, (const xmlChar *)"img")){
-                                       temp_xml_string = xmlGetProp(child_node3, (const xmlChar*)"src");
-                                       delimiter = strrchr((char*)temp_xml_string, '/');
-                                       if(delimiter){
-                                            delimiter++; /* delete '/' */
-                                            if (delimiter && !strcmp(delimiter,"day.gif"))
-                                                count_day++;
-                                            if (delimiter && !strcmp(delimiter,"night.gif"))
-                                                count_night++;
-                                       }
-                                       if (temp_xml_string)
-                                            xmlFree(temp_xml_string);
-                                    }
+   /* Create xpath evaluation context */
+   xpathCtx = xmlXPathNewContext(doc);
+   if(xpathCtx == NULL) {
+        fprintf(stderr,"Error: unable to create new XPath context\n");
+         return(-1);
+   }
+   /* Register namespaces from list (if any) */
+   xmlXPathRegisterNs(xpathCtx, (const xmlChar*)"html",
+                                (const xmlChar*)"http://www.w3.org/1999/xhtml");
 
-                                }
-                            }else
-                                if (temp_xml_string)
-                                    xmlFree(temp_xml_string);
-                        }
-                   }
-                }
+  /* Evaluate xpath expression */
+  xpathObj = xmlXPathEvalExpression((const xmlChar*)"/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/@title", xpathCtx);
+  
+
+  if(xpathObj == NULL) {
+        fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", "//*[@class='c0 day']/div/text()");
+        xmlXPathFreeContext(xpathCtx); 
+        return(-1);
+  }
+
+  nodes   = xpathObj->nodesetval;
+  size = (nodes) ? nodes->nodeNr : 0;
+    
+  xpathObj2 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0' and @title]/text()", xpathCtx);
+  xpathObj3 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c3']/text()", xpathCtx);
+  xpathObj4 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c1']/div/img/@src", xpathCtx);
+  xpathObj5 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c2']/span/text()", xpathCtx);
+  xpathObj6 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c4']/text()", xpathCtx);
+  xpathObj7 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c5']/div/text()", xpathCtx);
+  xpathObj8 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c5']/div/img/@title", xpathCtx);
+  xpathObj9 = xmlXPathEvalExpression("/html/body/div/div/div/div/div/div/div/table/tbody/tr/td[@class='c0']/following-sibling::*[@class='c6']/text()", xpathCtx);
+
+  /* fprintf(stderr, "Result (%d nodes):\n", size); */
+  for(i = 0; i < size; ++i) {
+      day = NULL;
+      /* Take UTC time: */
+      if (!nodes->nodeTab[i]->children->content)
+          continue;
+      temp_char = strstr(nodes->nodeTab[i]->children->content, "UTC: ");
+      if (temp_char && strlen(temp_char) >6)
+              temp_char = temp_char +5;
+
+      tmp_tm = get_date_for_hour_weather(temp_char);
+      memset(buff, 0, sizeof(buff));
+      setlocale(LC_TIME, "POSIX");
+      strftime(buff, sizeof(buff) - 1, "%b %d", &tmp_tm);
+      setlocale(LC_TIME, "");
+      tmp = forecast;
+      flag = FALSE;
+      night_flag = FALSE;
+      /* Check Day and Night */
+      if (xpathObj2->nodesetval->nodeTab[i]->content && (
+           !strcmp(xpathObj2->nodesetval->nodeTab[i]->content, "Ночь")||
+           !strcmp(xpathObj2->nodesetval->nodeTab[i]->content, "День"))
+          ){
+            if (!strcmp(xpathObj2->nodesetval->nodeTab[i]->content, "Ночь"))
+                night_flag = TRUE;
+            /* fprintf(stderr,"Day : %s\n", xpathObj2->nodesetval->nodeTab[i]->content); */
+      }else{
+          continue;
+      }
+      /* Look up this day in hash */
+      while(tmp){
+          day = (GHashTable*)tmp->data;
+          if (g_hash_table_lookup(day, "day_date") &&
+              !strcmp(g_hash_table_lookup(day,"day_date"), buff)){
+                  flag = TRUE;
+                  break;
+          }
+          tmp = g_slist_next(tmp);
+      }
+      /* added new day to hash */
+      if (!flag){
+          day = g_hash_table_new(g_str_hash, g_str_equal);
+          g_hash_table_insert(day, "day_date", g_strdup(buff));
+          strftime(buff, sizeof(buff) - 1, "%a", &tmp_tm);
+          g_hash_table_insert(day, "day_name", g_strdup(buff));
+          forecast = g_slist_append(forecast,(gpointer)day);
+      }
+     
+      if (day){
+         /* added temperature */
+         if (xpathObj3->nodesetval->nodeTab[i]->content){
+            /* fprintf (stderr, "temperature %s\n", xpathObj3->nodesetval->nodeTab[i]->content); */
+            if (night_flag){
+                g_hash_table_insert(day, "day_low_temperature", g_strdup(xpathObj3->nodesetval->nodeTab[i]->content));
+                g_hash_table_insert(day, "night_temperature", g_strdup(xpathObj3->nodesetval->nodeTab[i]->content));
+            }else{
+                g_hash_table_insert(day, "day_hi_temperature", g_strdup(xpathObj3->nodesetval->nodeTab[i]->content));
+                g_hash_table_insert(day, "day_temperature", g_strdup(xpathObj3->nodesetval->nodeTab[i]->content));
             }
-        }
-    }
-    /* Filling data */
-    count_of_div = 0;
-    count_of_div_temp = 0;
-    count_of_table = 0;
-    for(cur_node = root_node->children; cur_node; cur_node = cur_node->next){
-        if( cur_node->type == XML_ELEMENT_NODE ){
-             /*  fprintf(stderr,"cur_node %s\n",cur_node->name); */
-            /* get weather station data */
-            if(!xmlStrcmp(cur_node->name, (const xmlChar *) "body" ) ){
-                for(child_node = cur_node->children; child_node; child_node = child_node->next){
-                    if( child_node->type == XML_ELEMENT_NODE ){
-                        if (!xmlStrcmp(child_node->name, (const xmlChar *)"div") ){
-                            count_of_div ++;
-                            count_of_div_temp ++;
-                        }
-                        if (!xmlStrcmp(child_node->name, (const xmlChar *)"table") )
-                            count_of_table ++;
-                        if (count_of_div == 2 && count_of_table == 0){
-                            temp_xml_string = xmlNodeGetContent(child_node);
-                            if (temp_xml_string){
-                                /* fprintf(stderr,"Date %s\n", temp_xml_string); */
-                                tmp_tm = get_data_from_russia_data((char*)temp_xml_string);
-                                first_day = mktime(&tmp_tm) -3600*24;
-                                day_in_list = first_day+(count_day*3600*24);
-                                night_in_list = first_day+(count_night*3600*24);
-                                xmlFree(temp_xml_string);
-                            }
-                        }
-                        if (count_of_div > 2 && 
-                            !xmlStrcmp(child_node->name, (const xmlChar *)"div")){
-                            temp_xml_string = xmlGetProp(child_node, (const xmlChar*)"style");
-                            if (!xmlStrcmp(temp_xml_string, 
-                                           (const xmlChar*)"float: left; padding: 0 0 0 4px;"))
-                            {
-                                xmlFree(temp_xml_string);
-                                for(child_node3 = child_node->children; child_node3 != NULL; child_node3 = child_node3->next){
-                                    if (!xmlStrcmp(child_node3->name, (const xmlChar *)"img")){
-                                       temp_xml_string = xmlGetProp(child_node3, (const xmlChar*)"src");
-                                       delimiter = strrchr((char*)temp_xml_string, '/');
-                                       if(delimiter){
-                                            delimiter++; /* delete '/' */
-                                            if (delimiter && 
-                                                (!strcmp(delimiter,"day.gif")||
-                                                (!strcmp(delimiter,"night.gif"))
-                                                )){
-                                                memset(buff, 0, sizeof(buff));
-                                                setlocale(LC_TIME, "POSIX");
+         }
+         /* added icon */
+         if (xpathObj4->nodesetval->nodeTab[i]->children->content){
+            /* fprintf (stderr, "sdfff %s\n", xpathObj4->nodesetval->nodeTab[i]->children->content); */
+            temp_char = strrchr((char*)xpathObj4->nodesetval->nodeTab[i]->children->content, '/');
+            temp_char ++;
+            if (night_flag)
+                g_hash_table_insert(day, "night_icon", choose_hour_weather_icon(hash_for_icons, temp_char));
+            else
+                g_hash_table_insert(day, "day_icon", choose_hour_weather_icon(hash_for_icons, temp_char));
+         }
+         /* added text */
+         if (xpathObj5->nodesetval->nodeTab[i]->content){
+            /* fprintf (stderr, "sdfff %s\n", xpathObj5->nodesetval->nodeTab[i]->content); */
+            if (night_flag)
+                g_hash_table_insert(day, "night_title", 
+                                 g_strdup(hash_gismeteo_table_find(hash_for_translate, xpathObj5->nodesetval->nodeTab[i]->content, FALSE)));
+            else
+            g_hash_table_insert(day, "day_title", 
+                                 g_strdup(hash_gismeteo_table_find(hash_for_translate, xpathObj5->nodesetval->nodeTab[i]->content, FALSE)));
+         }
+         /* added pressure */
+         if (xpathObj6->nodesetval->nodeTab[i]->content){
+            pressure = atoi((char*)xpathObj6->nodesetval->nodeTab[i]->content);
+            pressure = pressure * 1.333224;
+            snprintf(buffer, sizeof(buffer)-1,"%i", pressure);
+            /* fprintf (stderr, "temperature %s\n", xpathObj6->nodesetval->nodeTab[i]->content); */
+            if (night_flag)
+               g_hash_table_insert(day, "night_pressure", g_strdup(buffer));
+            else
+                g_hash_table_insert(day, "day_pressure", g_strdup(buffer));
+         }
+         /* added wind speed */
+         if (xpathObj7->nodesetval->nodeTab[i]->content){
+            /* Normalize speed to km/h from m/s */
+            /* fprintf(stderr, "Wind  speed    %s\n", temp_buffer); */
+            speed = atoi (xpathObj7->nodesetval->nodeTab[i]->content);
+            speed = speed * 3600/1000;
+            sprintf(buffer, "%i", speed);
+            if (night_flag)
+                g_hash_table_insert(day, "night_wind_speed", g_strdup(buffer));
+            else
+                g_hash_table_insert(day, "day_wind_speed", g_strdup(buffer));
+         }
+         /* added wind direction */
+         if (xpathObj8->nodesetval->nodeTab[i]->children->content){
+             /* fprintf (stderr, "Wind direction: %s\n", xpathObj8->nodesetval->nodeTab[i]->children->content); */
+             snprintf(buffer, sizeof(buffer)-1,"%s", xpathObj8->nodesetval->nodeTab[i]->children->content);
+             /* Wind direction */
+             if (!strcoll(buffer, "З"))
+                  sprintf(buffer,"%s","W");
+             if (!strcoll(buffer, "Ю"))
+                  sprintf(buffer,"%s","S");
+             if (!strcoll(buffer, "В"))
+                  sprintf(buffer,"%s","E");
+             if (!strcoll(buffer, "С"))
+                  sprintf(buffer,"%s","N");
+             if (!strcoll(buffer, "ЮЗ"))
+                  sprintf(buffer,"%s","SW");
+             if (!strcoll(buffer, "ЮВ"))
+                  sprintf(buffer,"%s","SE");
+             if (!strcoll(buffer, "СЗ"))
+                  sprintf(buffer,"%s","NW");
+             if (!strcoll(buffer, "СВ"))
+                  sprintf(buffer,"%s","NE");
+             if (!strcoll(buffer, "безветрие"))
+                  sprintf(buffer,"%s","CALM");
+             if (!strcoll(buffer, "Ш"))
+                  sprintf(buffer,"%s","CALM");
 
-                                                if (!strcmp(delimiter,"day.gif"))
-                                                    gmt = gmtime(&day_in_list);
-                                                if (!strcmp(delimiter,"night.gif"))
-                                                    gmt = gmtime(&night_in_list);
-
-                                                memset(buff, 0, sizeof(buff));
-                                                strftime(buff, sizeof(buff) - 1, "%b %d", gmt);
-                                                setlocale(LC_TIME, "");
-                                                /* searching of a day */
-                                                tmp = forecast;
-                                                flag = FALSE;
-                                                while(tmp){
-                                                    day = (GHashTable*)tmp->data;
-                                                    if (g_hash_table_lookup(day, "day_date") &&
-                                                        !strcmp(g_hash_table_lookup(day,"day_date"), buff)){
-                                                            flag = TRUE;
-                                                            break;
-                                                    }
-                                                    tmp = g_slist_next(tmp);
-                                                }
-                                                if (!flag){
-                                                    day = g_hash_table_new(g_str_hash, g_str_equal);
-                                                    g_hash_table_insert(day, "day_date", g_strdup(buff));
-                                                    strftime(buff, sizeof(buff) - 1, "%a", gmt);
-                                                    g_hash_table_insert(day, "day_name", g_strdup(buff));
- 
-                                                    forecast = g_slist_prepend(forecast,(gpointer)day);
-                                                }
-                                                if (day){
-                                                    if (!strcmp(delimiter,"day.gif")){
-                                                        fill_day(child_node, day, DAY, hash_for_translate, hash_for_icons);
-                                                        day_in_list = day_in_list - 3600*24;
-                                                    }
-                                                    if (!strcmp(delimiter,"night.gif")){
-                                                        fill_day(child_node, day, NIGHT, hash_for_translate, hash_for_icons);
-                                                        night_in_list = night_in_list - 3600*24;
-                                                    }
-
-                                                }
-                                            }
-                                       }
-                                       if (temp_xml_string)
-                                            xmlFree(temp_xml_string);
-                                    }
-                                }
-                            }else
-                                if (temp_xml_string)
-                                        xmlFree(temp_xml_string);
-                        }
-                   }
-                }
-                g_hash_table_insert(data, "forecast", (gpointer)forecast);
+             if (night_flag)
+                g_hash_table_insert(day, "night_wind_title", g_strdup(buffer));
+             else
+                g_hash_table_insert(day, "day_wind_title", g_strdup(buffer));
+         }
+         /* added humidity */
+         if (xpathObj9->nodesetval->nodeTab[i]->content){
+            /* fprintf (stderr, "temperature %s\n", xpathObj1->nodesetval->nodeTab[0]->content); */
+            if (night_flag){
+                g_hash_table_insert(day, "night_humidity", g_strdup(xpathObj9->nodesetval->nodeTab[i]->content));
+            }else{
+                g_hash_table_insert(day, "day_humidity", g_strdup(xpathObj9->nodesetval->nodeTab[i]->content));
             }
-        }
-    }
-    g_hash_table_destroy(hash_for_translate);
-    g_hash_table_destroy(hash_for_icons);
-    return count_day;
+         }
+
+      }
+  }	
+  /* Cleanup */
+  if (xpathObj)
+    xmlXPathFreeObject(xpathObj);
+  if (xpathObj2)
+    xmlXPathFreeObject(xpathObj2);
+  if (xpathObj3)
+    xmlXPathFreeObject(xpathObj3);
+  if (xpathObj4)
+    xmlXPathFreeObject(xpathObj4);
+  if (xpathObj5)
+    xmlXPathFreeObject(xpathObj5);
+  if (xpathObj6)
+    xmlXPathFreeObject(xpathObj6);
+  if (xpathObj7)
+    xmlXPathFreeObject(xpathObj7);
+  if (xpathObj8)
+    xmlXPathFreeObject(xpathObj8);
+  if (xpathObj9)
+    xmlXPathFreeObject(xpathObj9);
+  if (xpathCtx)
+    xmlXPathFreeContext(xpathCtx); 
+
+  g_hash_table_insert(data, "forecast", (gpointer)forecast);
+  g_hash_table_destroy(hash_for_translate);
+  g_hash_table_destroy(hash_for_icons);
+
+  return size/4;
 }
 /*******************************************************************************/
 void
