@@ -7,16 +7,16 @@
  * 	for the code
  *
  * This software is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
+ * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ *  General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
+ * You should have received a copy of the GNU  General Public
  * License along with this software; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
@@ -28,12 +28,15 @@
 #include <gtk/gtk.h>
 #include <libxml/parser.h>
 #include <libxml/HTMLparser.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
 #include <stdlib.h>
 #include <glib.h>
 #include <locale.h>
 /*******************************************************************************/
 gint get_station_weather_data(const gchar *station_id_with_path, GHashTable *data, gboolean get_detail_data);
-gint parse_xml_data(const gchar *station_id, xmlNode *root_node, GHashTable *data);
+//gint parse_xml_data(const gchar *station_id, xmlNode *root_node, GHashTable *data);
+gint parse_xml_data(const gchar *station_id, htmlDocPtr doc, GHashTable *data);
 gint parse_xml_detail_data(const gchar *station_id, xmlNode *root_node, GHashTable *data);
 //void fill_detail_data(xmlNode *root_node, GHashTable *day, gint part_of_day, GHashTable *hash_for_translate, GHashTable *hash_for_icons)
 //void  fill_day (xmlNode *root_node, GHashTable *day, gint part_of_day, GHashTable *hash_for_translate, GHashTable *hash_for_icons)
